/*  cssed (c) Iago Rubio 2003, 2004 - A tiny CSS editor.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifndef PLAT_GTK
#   define PLAT_GTK 1
#endif

//#define DEBUG 
//#define DEBUG_CONTROL_NOTIFICATIONS 
#define WITH_FOLDING
// this is the line numbers width template
#define MARGIN_LINES_WIDTH_STRING "_9999"
// markers margin width in pixels
#define MARGIN_MARKERS_WIDTH_PIXELS 12
// folding margin width in pixels
#define MARGIN_FOLDING_WIDTH_PIXELS 16


#include <gtk/gtk.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <libgen.h>	

#ifdef WITH_TERMINAL
 #include <vte/vte.h>
#endif

#include "cssedwindow.h"
#include "callbacks.h"
#include "cssdialogs-interface.h"
#include "interface.h"
#include "support.h"
#include "document.h"

#include <Scintilla.h>
#include <SciLexer.h>
#include <ScintillaWidget.h>

/* some prototipes will be moved to sciwrappers.h and functions to sciwrappers.c*/
void 				on_editor_notification		(GtkWidget* editor,gint scn, gpointer lscn, gpointer user_data);
void 				sci_set_css_style 			(ScintillaObject * sci);
ScintillaObject* 	sci_get_from_document		(Tcssed_doc* doc);
ScintillaObject* 	sci_get_from_notebook_page	(GtkNotebook* notebook, gint 		page_number);
void 				sci_set_line_numbers		(ScintillaObject* sci,  gboolean 	set );
void 				sci_set_keyworkds 			(ScintillaObject* sci,  gchar* 		keywords);
void 				sci_add_text 				(ScintillaObject* sci,  gchar* 		text);
void 				sci_undo					(ScintillaObject* sci);
void 				sci_redo					(ScintillaObject* sci);
void 				sci_empty_undo_buffer		(ScintillaObject* sci);
void 				sci_end_undo_action			(ScintillaObject* sci);
void 				sci_start_undo_action		(ScintillaObject* sci);
void				sci_set_visible_eols		(ScintillaObject* sci, gboolean set );
gboolean			sci_get_visible_eols		(ScintillaObject* sci);
void				sci_set_lines_wrapped		(ScintillaObject* sci, gboolean set );
gboolean 			sci_get_lines_wrapped		(ScintillaObject* sci);
void				sci_set_visible_white_spaces(ScintillaObject* sci, gboolean set );
gboolean 			sci_get_visible_white_spaces(ScintillaObject* sci);
gint				sci_get_eol_mode			(ScintillaObject* sci);
void 				sci_convert_eols			(ScintillaObject* sci, gint eolmode);
void 				sci_set_eol_mode			(ScintillaObject* sci, gint eolmode);
void 				sci_zoom_in					(ScintillaObject* sci);
void 				sci_zoom_out				(ScintillaObject* sci);
void 				sci_zoom_off				(ScintillaObject* sci);
void				sci_set_marker_at_line		(ScintillaObject* sci, gint line_number, gboolean set, gint marker );
gboolean 			sci_is_marker_set_at_line	(ScintillaObject* sci, gint line, gint marker);
gboolean 			sci_marker_next				(ScintillaObject* sci, gint line, gint marker_mask);
gboolean 			sci_marker_prev				(ScintillaObject* sci, gint line, gint marker_mask);

gint 				sci_get_line_from_position	(ScintillaObject* sci, gint position);
gint 				sci_get_position_from_line	(ScintillaObject* sci, gint line );
gint 				sci_get_current_position	(ScintillaObject* sci);
void 				sci_set_current_position	(ScintillaObject* sci, gint position);
void 				sci_set_current_line		(ScintillaObject* sci, gint line);

void 				sci_cut						(ScintillaObject* sci);
void 				sci_copy					(ScintillaObject* sci);
void 				sci_paste					(ScintillaObject* sci);

gint				sci_get_selection_start		(ScintillaObject* sci);
gint				sci_get_selection_end		(ScintillaObject* sci);
void 				sci_replace_sel				(ScintillaObject* sci, gchar* text);

gint				sci_get_length				(ScintillaObject* sci);
void				sci_get_text				(ScintillaObject* sci,gint len,gchar* text);

void				sci_get_line				(ScintillaObject* sci, gint line, gchar* text);
gint 				sci_line_length				(ScintillaObject* sci, gint line);
gint				sci_get_line_count			( ScintillaObject* sci );
void 				sci_get_xy_from_position	(ScintillaObject* sci,gint pos, gint* x, gint* y);

void 				sci_set_undo_collection		(ScintillaObject* sci, gboolean set);
gboolean			sci_get_undo_collection		(ScintillaObject* sci);

void 				sci_set_selection_start		(ScintillaObject* sci, gint position);
void				sci_set_selection_end		(ScintillaObject* sci, gint position);
gint				sci_get_line_end_from_position	(ScintillaObject* sci, gint position);

void 				sci_toggle_fold				(ScintillaObject* sci, gint line);
gboolean			sci_get_line_is_visible		(ScintillaObject* sci, gint line);
void				sci_ensure_line_is_visible	(ScintillaObject* sci, gint line);
gint				sci_get_fold_level			(ScintillaObject* sci, gint line);

void 				sci_set_folding_margin_visible (ScintillaObject * sci, gboolean set );
gboolean			sci_get_folding_margin_visible(ScintillaObject * sci);
gboolean			sci_get_fold_expanded(ScintillaObject* sci, gint line);
void				sci_colourise(ScintillaObject* sci, gint start, gint end);
/*
	create an empty doc with a scintilla editor 
*/

Tcssed_doc *
cssed_create_new_doc ( Tcssed_window* window )
{
	GtkWidget *editor;
	ScintillaObject *sci;
	Tcssed_doc *newdoc;
	gint sciid;
	Tcssed_cfg* cfg;
	
	newdoc = g_malloc (sizeof (Tcssed_doc));
	cfg = window->config;
	
	editor = scintilla_new ();
	sci = SCINTILLA (editor);
	sciid = get_new_sci_number ();
	scintilla_set_id (sci, sciid);

#ifdef DEBUG
	g_print ("document.c:cssed_create_new_doc() - created doc\n\teditor\t%x\n\tsiid\t%d\n", editor, sciid);
	g_print ("document.c:cssed_create_new_doc() - in list %d\n\n", g_list_length (window->document_list));
#endif
	g_signal_connect( editor, "sci-notify", G_CALLBACK(on_editor_notification), (gpointer) window );
	/* This is unused now but will be used in the future to track function key press events and others
	 * The callback func is in callbacks.c 
	g_signal_connect( editor, "key-press-event", G_CALLBACK(on_editor_key_press_event), (gpointer) window );
	*/
	g_signal_connect( editor, "grab-focus" , G_CALLBACK(on_editor_grab_focus), (gpointer) window );

	gtk_widget_set_usize (editor, 500, 300);
	newdoc->editor = editor;					  // scintilla editor
	newdoc->scid = sciid;						  // scintilla id
	newdoc->notebook = window->notebook_docs;	  // main notebook where this doc is attached
	newdoc->filename = NULL;    				  // full path to file if exists
	newdoc->modified = FALSE;					  // fresh document, so unmodified
	newdoc->filetype = CSSED_FILETYPE_CSS;		  // one of the supported filetype css, html, text, bynary or unknow
	newdoc->autocompletion = cfg->autocompletion; // autocompletion from config
	newdoc->window = window;
	
	document_set_css_style( newdoc );
	document_set_font( newdoc );
	document_set_current_font_size( newdoc );
	document_set_default_configuration( newdoc );
	document_set_undo_collection( newdoc, TRUE );
	document_start_undo_action(newdoc);
	
	return newdoc;
}

void
document_save(Tcssed_doc* doc)
{
	Tcssed_window* window;
	GtkWidget *file_selector;
	gchar *doc_buffer;
	gint doclen;
	size_t nwriten;
	FILE *f;
	gchar *msg;
	
	window = CSSED_WINDOW(doc->window);
	
	if (doc->filename == NULL)
	{
		file_selector = create_save_fileselection (window);
		gtk_widget_show (file_selector);
	}
	else
	{
#ifdef DEBUG
		g_print ("Trying to save a document called %s\n",
			 doc->filename);
#endif
		f = fopen (doc->filename, "w");
		if (f == NULL)
		{
			msg = g_strdup_printf
				(_("Unable to open the file\n%s\nfor writing."),
				 doc->filename);
			cssed_error_message ( _("Can't open file"),msg );
			g_free (msg);
		}
		else
		{
			doclen = document_get_length (doc);
			doc_buffer = g_malloc0 (++doclen);
			document_get_text (doc, doclen, doc_buffer);
			
			nwriten =
				fwrite (doc_buffer, sizeof (gchar), strlen(doc_buffer), f);
			if ( nwriten != strlen(doc_buffer) )
			{
				cssed_error_message
					(_("Error while writing to file"),
					 _("There is a mismatch between the file size\nand the bytes written.\nThe file can be damaged.") );
			}
			document_set_modified (doc, FALSE);
			g_free (doc_buffer);
			fclose (f);
		}
	}
}

/*  create an empty doc with an scintilla editor and
	attach to the notebook passed as argument, puts the
	string title on notebook label.
*/
void
create_and_attach_new_doc (Tcssed_window* window , const gchar * title)
{
	Tcssed_doc *newdoc;
	GtkWidget *label;
	//GtkTooltips* doc_tips;
	
	newdoc = cssed_create_new_doc (window);
	if (newdoc == NULL)
	{ // FIXME say something to user
#ifdef DEBUG
		g_print ("document.c:create_and_attach_new_doc() - passed null doc !!!\n\n");
#endif
	}
	else
	{
#ifdef DEBUG
		g_print ("document.c:create_and_attach_new_doc() - doc passed %x\n", newdoc);
		g_print ("document.c:create_and_attach_new_doc() - in list %d\n\n", g_list_length (window->document_list));
#endif
		window->document_list = g_list_append (window->document_list, newdoc);
		gtk_widget_show (newdoc->editor);
		//doc_tips = gtk_tooltips_new();
		label = gtk_label_new (title);
		//gtk_tooltips_set_tip  (doc_tips,label,"Untitled","");
		gtk_notebook_append_page ((GtkNotebook *) window->notebook_docs,  newdoc->editor, label);
		gtk_widget_queue_draw (GTK_WIDGET (window->notebook_docs));
	}
}

void
create_and_attach_new_named_doc_from_fileselector_cb (GtkButton * button, gpointer user_data)
{
	GtkWidget* file_selector;
	Tcssed_fileselector_data* data;
	Tcssed_window* window;			
	const gchar *selected_filename;
	gchar* filename;

	data = (Tcssed_fileselector_data*) user_data;
	window = (Tcssed_window *) data->main_window;
	file_selector = GTK_WIDGET (data->fileselector);

	selected_filename =	gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
	filename = g_strdup( selected_filename ); // avoid compiler warnings
	gtk_widget_destroy (gtk_widget_get_toplevel (file_selector));
	create_and_attach_new_named_doc( window, filename );
	g_free( filename );

	g_free (data);
}

void
create_and_attach_new_named_doc(Tcssed_window* window, gchar* filename)
{
	Tcssed_doc* newdoc;
	GtkWidget* label;
	GtkWidget* notebook;
	gint pnum;
	struct stat _stat;
	gint filedes;
	gint _retval;
	void* map;
	gchar* ptr;
	gchar* block_end;
	gchar* line_start;
	gchar* line_buffer;
	gint line_size;
	gchar* file_name;
	gchar* filenamecp;
	gchar* errormsg;
	//GtkTooltips *doc_tips;
	
	notebook = GTK_WIDGET (window->notebook_docs);


#ifdef DEBUG
	g_print ("document.c - create_and_attach_new_named_doc\n");
	g_print ("\tFfilename: %s\n\n", filename);
#endif

	if (!g_file_test
	    (filename, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
	{
		errormsg = g_strdup_printf(_("File %s\ndon't exist or is not a regular file."),filename);
		cssed_error_message (errormsg,"Error opening file");
		g_free(errormsg);
		return;
	}
	_retval = stat (filename, &_stat);
	if (_retval != 0)
	{
		errormsg = g_strdup_printf(_("Can't get information about file\n%s."),filename);
		cssed_error_message (errormsg,_("Error opening file"));
		g_free(errormsg);
		return;
	}
	if ((filedes = open (filename, O_RDWR)) < 0)
	{
		errormsg = g_strdup_printf(_("Can't open the file\n%s."),filename);
		cssed_error_message (errormsg,_("Error opening file") );
		g_free(errormsg);
		return;
	}
	if ((map =
	     mmap (0, _stat.st_size, PROT_READ, MAP_SHARED, filedes,
		   0)) == MAP_FAILED)
	{
		errormsg = g_strdup_printf(_("Cannot map to memory the file\n%s."),filename);
		cssed_error_message (errormsg,_("Error opening file") );
		close (filedes);
		g_free(errormsg);
		return;
	}

	ptr = map;		// alocate the pointer at start of memory block
	block_end = map + _stat.st_size;	     // end of memory block
	line_start = map;	                     // start of first line

	newdoc = cssed_create_new_doc (window);
	window->document_list = g_list_append (window->document_list, newdoc);
	
	if (newdoc == NULL)
	{
#ifdef DEBUG /* FIXME; say something to user */
		g_print ("document.c create_and_attach_new_named_doc() \n\tERR _ Passed null doc !!!\n");
#endif
		close (filedes);
		munmap(map, _stat.st_size);
		return;
	}
	else
	{
#ifdef DEBUG
		g_print ("document.c create_and_attach_new_named_doc() \n\tDoc passed %p\n", newdoc);
		g_print ("document.c create_and_attach_new_named_doc() \n\tin list %d\n", g_list_length (window->document_list));
#endif
		// add the file to the editor 
		line_size = 0;
		// if empty just return
		if( _stat.st_size == 0 ){
			errormsg = g_strdup_printf("open file: the file '%s' is empty",filename);
			document_write_error_to_program_output(newdoc, errormsg,ERROR_COLOR_STRING);
			g_free(errormsg);
		}else{
			while( ptr <= block_end ){
				if( ( (char) *ptr == '\n' ) || (ptr == block_end) ){
					// add line
					line_buffer = (char*) calloc( line_size + 1, sizeof(char));
					if( line_buffer == NULL ){ /* FIXME; say something to user */
#ifdef DEBUG
					g_print ("document.c create_and_attach_new_named_doc() \n\tCan't reserve mem, calloc returned %p\n", line_buffer);
#endif
						close (filedes);
						munmap(map, _stat.st_size);
						return;
					}else{		
						line_buffer = strncpy(line_buffer,line_start , line_size * sizeof(char));
						document_add_text_line( newdoc,(char*)  line_buffer);
						free(line_buffer);
					}
					line_size = 0;
					line_start = ptr + 1;
				}else{
					line_size++;
				}
				ptr++;
			}
		}
		filenamecp = (char*) g_strdup( filename );		
		file_name = basename( filenamecp );
		
		newdoc->notebook = notebook;
		newdoc->filename = filenamecp;
		newdoc->filetype = CSSED_FILETYPE_CSS; // this must change when new styles appear
		newdoc->modified = FALSE;
		
		//doc_tips = gtk_tooltips_new ();
		gtk_widget_show (newdoc->editor);
		label = gtk_label_new (file_name);
		//gtk_tooltips_set_tip  (doc_tips,label,file_name,"");
		gtk_notebook_append_page ((GtkNotebook *) notebook, newdoc->editor, label);
#ifdef GTK_IS_2_2
		pnum = gtk_notebook_get_n_pages (GTK_NOTEBOOK(notebook));
#else
		pnum = g_list_length(GTK_NOTEBOOK(notebook)->children);
#endif
		gtk_notebook_set_current_page   (GTK_NOTEBOOK(notebook), --pnum);		
		gtk_widget_queue_draw (GTK_WIDGET (notebook));
		document_colourise_all(newdoc);
		document_empty_undo_buffer(newdoc);
		document_set_modified(newdoc, FALSE);
	}
	close (filedes);
	munmap(map, _stat.st_size);
}

/* create one alone editor used as a callback from create_window
	NEDDS FIX don't use in other created documents. 
	use create_and_attach_new_doc() instead
*/
GtkWidget *
create_initial_blank_editor (Tcssed_window* window)
{
	Tcssed_doc *newdoc;

	newdoc = cssed_create_new_doc (window);
	window->document_list = g_list_append (window->document_list, newdoc);

#ifdef DEBUG
	g_print ("document.c:create_initial_blank_editor ()\n\tInitial editor created\n");
	g_print ("document.c:create_initial_blank_editor ()\n\tin list %d\n", g_list_length (window->document_list));
#endif

	return newdoc->editor;
}

/*  Get diferent - incrementally - integers as sci ids
	so, right now ,we are limited in docs opened in a
	single editing session. Even if you closed them.
	NEEDS FIX 
*/
int
get_new_sci_number ()
{
	static int initialized = 0;
	static unsigned int count;

	if (initialized != 1)
	{
		initialized = 1;
		count = 0;
		return ++count;	// id 0 created on startup
	}
	else
	{
		return ++count;
	}
}
/* validator arrow */
void
document_set_validator_arrow_at_line( Tcssed_doc* doc, gint line )
{
	ScintillaObject * sci;	
	gint pos;
	sci =  sci_get_from_document( doc );
	
	SSM(sci, SCI_MARKERDELETEALL,1,0);
	SSM(sci, SCI_MARKERADD, line-1, 1);
	pos = document_get_position_from_line( doc, --line);
	document_ensure_line_is_visible( doc, line );
	document_set_current_position(doc, pos);
}
void
document_clear_validator_arrow( Tcssed_doc* doc )
{
	ScintillaObject * sci;	
	gint pos;
	sci =  sci_get_from_document( doc );
	
	SSM(sci, SCI_MARKERDELETEALL,1,0);
}
/* scroll to the caret 
	when you adds text, to the scintilla control, if the
	edge bound of text is down the editor window, no automatic
	scrolling is performed so you must call this func
	to scroll to the caret before text insertion.
*/
void 
document_scroll_to_caret( Tcssed_doc* doc )
{
	ScintillaObject * sci;
	sci = SCINTILLA( doc->editor );	
	SSM (sci, SCI_SCROLLCARET, 0, 0);
}
/* recolourise the document from start to end */
void				
document_colourise(Tcssed_doc* doc, gint start, gint end)
{
	ScintillaObject * sci;	
	sci = (ScintillaObject *) sci_get_from_document(  doc );	
	sci_colourise( sci, start, end );
}
void				
document_colourise_all(Tcssed_doc* doc)
{
	gint end;
	end = document_get_length( doc );
	document_colourise(doc , 0, end);
}
void				
document_style_clear_all(Tcssed_doc* doc) // no wrapper for this yet
{
	ScintillaObject * sci;	
	sci = (ScintillaObject *) sci_get_from_document(  doc );	
	SSM(sci, SCI_STYLECLEARALL, 0 , 0);
}

void 
document_set_css_style(Tcssed_doc* doc)
{
	ScintillaObject * sci;
	Tcssed_window* window;
	Tcssed_cfg* cfg;

	window = CSSED_WINDOW(doc->window);
	cfg = window->config;	
	sci = SCINTILLA( doc->editor );
	//sci_set_css_style (sci);

	SSM (sci, SCI_STYLECLEARALL, 0, 0);
	SSM (sci, SCI_SETLEXER, SCLEX_CSS, 0);
	
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_DEFAULT, cfg->DEFAULT_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_DEFAULT, cfg->DEFAULT_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_DEFAULT, cfg->DEFAULT_bold); 
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_TAG, cfg->TAG_fore_color);
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_TAG, cfg->TAG_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_TAG, cfg->TAG_bold);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_PSEUDOCLASS, cfg->PSEUDOCLASS_fore_color);
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_PSEUDOCLASS, cfg->PSEUDOCLASS_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_PSEUDOCLASS, cfg->PSEUDOCLASS_bold);	
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_UNKNOWN_PSEUDOCLASS, cfg->UNKNOWN_PSEUDOCLASS_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_UNKNOWN_PSEUDOCLASS, cfg->UNKNOWN_PSEUDOCLASS_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_UNKNOWN_PSEUDOCLASS, cfg->UNKNOWN_PSEUDOCLASS_bold);		
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_OPERATOR, cfg->OPERATOR_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_OPERATOR, cfg->OPERATOR_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_OPERATOR, cfg->OPERATOR_bold);	
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_IDENTIFIER, cfg->IDENTIFIER_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_IDENTIFIER, cfg->IDENTIFIER_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_IDENTIFIER, cfg->IDENTIFIER_bold);	
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_UNKNOWN_IDENTIFIER, cfg->UNKNOWN_IDENTIFIER_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_UNKNOWN_IDENTIFIER, cfg->UNKNOWN_IDENTIFIER_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_UNKNOWN_IDENTIFIER, cfg->UNKNOWN_IDENTIFIER_bold);		
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_VALUE, cfg->VALUE_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_VALUE, cfg->VALUE_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_VALUE, cfg->VALUE_bold);	
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_COMMENT, cfg->COMMENT_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_COMMENT, cfg->COMMENT_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_COMMENT, cfg->COMMENT_bold);	
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_IMPORTANT, cfg->IMPORTANT_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_IMPORTANT, cfg->IMPORTANT_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_IMPORTANT, cfg->IMPORTANT_bold);	
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_DIRECTIVE, cfg->DIRECTIVE_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_DIRECTIVE, cfg->DIRECTIVE_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_DIRECTIVE, cfg->DIRECTIVE_bold);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_DOUBLESTRING, cfg->DOUBLESTRING_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_DOUBLESTRING, cfg->DOUBLESTRING_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_DOUBLESTRING, cfg->DOUBLESTRING_bold);	
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_SINGLESTRING, cfg->SINGLESTRING_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_SINGLESTRING, cfg->SINGLESTRING_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_SINGLESTRING, cfg->SINGLESTRING_bold);		
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_CLASS, cfg->CLASS_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_CLASS, cfg->CLASS_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_CLASS, cfg->CLASS_bold);	
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_ID, cfg->ID_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_ID, cfg->ID_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_ID, cfg->ID_bold);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_VALID_VALUE, cfg->VALID_VALUE_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_VALID_VALUE, cfg->VALID_VALUE_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_VALID_VALUE, cfg->VALID_VALUE_bold);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_FUNCTION, cfg->FUNCTION_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_FUNCTION, cfg->FUNCTION_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_FUNCTION, cfg->FUNCTION_bold);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_NUMBER, cfg->NUMBER_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_NUMBER, cfg->NUMBER_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_NUMBER, cfg->NUMBER_bold);	
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_UNIT, cfg->UNIT_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_UNIT, cfg->UNIT_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_UNIT, cfg->UNIT_bold);	
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_COLOR, cfg->COLOR_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_COLOR, cfg->COLOR_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_COLOR, cfg->COLOR_bold);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_HEXACOLOR, cfg->HEXACOLOR_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_HEXACOLOR, cfg->HEXACOLOR_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_HEXACOLOR, cfg->HEXACOLOR_bold);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_ATTR_MATCH, cfg->ATTR_MATCH_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_ATTR_MATCH, cfg->ATTR_MATCH_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_ATTR_MATCH, cfg->ATTR_MATCH_bold);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_LANGUAGE, cfg->LANGUAGE_fore_color); 
	SSM (sci, SCI_STYLESETBACK, SCE_CSS_LANGUAGE, cfg->LANGUAGE_back_color); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_LANGUAGE, cfg->LANGUAGE_bold);
}
void
document_set_font_by_name(Tcssed_doc* doc, gchar* font_name)
{
	ScintillaObject * sci;
	Tcssed_window* window;

	window = CSSED_WINDOW(doc->window);
	sci = SCINTILLA( doc->editor );
	
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_DEFAULT,(sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_TAG, (sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_PSEUDOCLASS,(sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_UNKNOWN_PSEUDOCLASS,(sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_OPERATOR,(sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_IDENTIFIER, (sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_UNKNOWN_IDENTIFIER, (sptr_t) font_name );  
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_VALUE, (sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_COMMENT, (sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_IMPORTANT,(sptr_t) font_name );  
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_DIRECTIVE, (sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_DOUBLESTRING, (sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_SINGLESTRING,(sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_CLASS, (sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_ID, (sptr_t) font_name );  
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_VALID_VALUE, (sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_FUNCTION, (sptr_t) font_name );  
	SSM (sci, SCI_STYLESETFONT,SCE_CSS_NUMBER, (sptr_t) font_name );  
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_UNIT, (sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_COLOR, (sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_HEXACOLOR, (sptr_t) font_name ); 
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_ATTR_MATCH, (sptr_t) font_name );  
	SSM (sci, SCI_STYLESETFONT, SCE_CSS_LANGUAGE, (sptr_t) font_name ); 
}

void
document_set_font(Tcssed_doc* doc)
{
	Tcssed_window* window;
	Tcssed_cfg* cfg;
	
	window = CSSED_WINDOW(doc->window);
	cfg = window->config;
	document_set_font_by_name(doc, cfg->font);		
}

void
document_set_current_font_size(Tcssed_doc* doc)
{
	Tcssed_window* window;
	Tcssed_cfg* cfg;
	
	window = CSSED_WINDOW(doc->window);
	cfg = window->config;
	document_set_font_size(doc, cfg->font_size);		
}

void
document_set_font_size(Tcssed_doc* doc, gint size)
{
	ScintillaObject * sci;
	Tcssed_window* window;

	window = CSSED_WINDOW(doc->window);
	sci = SCINTILLA( doc->editor );
	
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_DEFAULT, size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_TAG, size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_PSEUDOCLASS, size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_UNKNOWN_PSEUDOCLASS, size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_OPERATOR, size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_IDENTIFIER,  size );
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_UNKNOWN_IDENTIFIER,  size );  
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_VALUE,  size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_COMMENT,  size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_IMPORTANT, size );  
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_DIRECTIVE,  size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_DOUBLESTRING,  size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_SINGLESTRING, size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_CLASS, size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_ID, size );  
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_VALID_VALUE, size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_FUNCTION, size );  
	SSM (sci, SCI_STYLESETSIZE,SCE_CSS_NUMBER, size );  
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_UNIT, size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_COLOR, size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_HEXACOLOR, size ); 
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_ATTR_MATCH, size );  
	SSM (sci, SCI_STYLESETSIZE, SCE_CSS_LANGUAGE, size ); 	
}


void
document_set_default_configuration( Tcssed_doc* doc )
{
	ScintillaObject * sci;
	Tcssed_window* window;
	Tcssed_cfg* cfg;

	window = CSSED_WINDOW(doc->window);
	cfg = window->config;	
	
	sci = SCINTILLA( doc->editor );
	SSM (sci, SCI_SETTABWIDTH, 4, 0);
	SSM (sci, SCI_SETREADONLY, 0, 0);
	SSM (sci, SCI_SETMARGINTYPEN, 0, SC_MARGIN_NUMBER);
	SSM (sci, SCI_SETMARGINTYPEN, 1, SC_MARGIN_SYMBOL);
	SSM (sci, SCI_SETMARGINWIDTHN, 1, MARGIN_MARKERS_WIDTH_PIXELS); // always opened
	SSM (sci, SCI_SETMARGINSENSITIVEN,1, TRUE);
	SSM (sci, SCI_MARKERDEFINE,0, SC_MARK_ROUNDRECT); // marker 0 for bookmarks	
	SSM (sci, SCI_MARKERDEFINE,1, SC_MARK_ARROWS); // marker 1 for validator
	SSM (sci, SCI_MARKERSETFORE,1, 0x0000ff);
	SSM (sci,SCI_SETUNDOCOLLECTION,1,0);
	SSM (sci,SCI_SETMODEVENTMASK, SC_MODEVENTMASKALL ,0); 
#ifdef WITH_FOLDING
	SSM (sci, SCI_SETPROPERTY, (sptr_t) "fold", (sptr_t) "1");	
	SSM (sci, SCI_SETPROPERTY, (sptr_t) "fold.comment", (sptr_t) "1");	
	SSM (sci, SCI_SETPROPERTY, (sptr_t) "fold.compact", (sptr_t) "1");	
 	SSM (sci, SCI_SETFOLDFLAGS, 16,0);
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDEROPEN, SC_MARK_BOXMINUS);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDEROPEN, 0xffffff);
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDEROPEN, 0x000000);
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDER, SC_MARK_BOXPLUS);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDER, 0xffffff);
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDER, 0x000000);	
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDERSUB, SC_MARK_VLINE);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDERSUB, 0xffffff);
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDERSUB, 0x000000);	
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDERTAIL, SC_MARK_LCORNER);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDERTAIL, 0xffffff);	
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDERTAIL, 0x000000);	
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDEREND, SC_MARK_BOXPLUSCONNECTED);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDEREND, 0xffffff);
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDEREND, 0x000000);	
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDEROPENMID, SC_MARK_BOXMINUSCONNECTED);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDEROPENMID, 0xffffff);
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDEROPENMID, 0x000000);	
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_TCORNER);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDERMIDTAIL, 0xffffff);
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDERMIDTAIL, 0x000000);	
	SSM (sci, SCI_SETMARGINSENSITIVEN,2, TRUE);
	SSM (sci, SCI_SETMARGINMASKN,2, SC_MASK_FOLDERS);

	if( cfg->folding ){
		SSM (sci, SCI_SETMARGINWIDTHN, 2, MARGIN_FOLDING_WIDTH_PIXELS );
	}else{
		SSM (sci, SCI_SETMARGINWIDTHN, 2, 0 );
	}
#endif	 
	if( cfg->linenumbers ){
			SSM (sci, SCI_SETMARGINWIDTHN, 0, 
				SSM(sci, SCI_TEXTWIDTH, STYLE_LINENUMBER,
				(sptr_t) MARGIN_LINES_WIDTH_STRING));
	}else{
		SSM (sci, SCI_SETMARGINWIDTHN, 0, 0 );	
	}

	if( cfg->lineendings ){
		sci_set_visible_eols( sci, TRUE );
	}
	
	if( cfg->whitespaces ){
		sci_set_visible_white_spaces(sci, TRUE);
	}
	
	if( cfg->lineswraped ){
		sci_set_lines_wrapped( sci, TRUE );
	}
		
	sci_set_keyworkds (sci, (gchar*) window->keywords_list);
#ifdef DEBUG
	g_print("document.c: document_set_css_style() - Keywords used\n%s\n\n",window->keywords_list);
#endif
}
Tcssed_doc*
document_get_from_editor( Tcssed_window* window, GtkWidget* editor )
{
	GList* tmplist;
	Tcssed_doc* tmpdoc;	
	
	tmplist = g_list_first(window->document_list);
	while(tmplist){	
		tmpdoc = (Tcssed_doc*) tmplist->data;
		if( editor == (GtkWidget*) tmpdoc->editor ){
			return tmpdoc;
		}else{
			tmplist = g_list_next(tmplist);
		}
	}
	// no doc in list, if it reachs here...  :-(
	return NULL;	
}
Tcssed_doc*
document_get_from_notebook_page_num(Tcssed_window* window, gint page_number)
{
	GtkWidget* editor;
	Tcssed_doc* tmpdoc;
	GtkWidget* notebook;
	
	notebook = window->notebook_docs;
	
	editor = gtk_notebook_get_nth_page (GTK_NOTEBOOK(notebook), page_number);
	return document_get_from_editor( window, editor );		
}

Tcssed_doc*
document_get_current(Tcssed_window* window)
{
	GtkWidget* notebook;
	Tcssed_doc* tmpdoc;
	gint current_index;
	
	notebook = window->notebook_docs;
	
	current_index =  gtk_notebook_get_current_page   ( GTK_NOTEBOOK(notebook) );
	tmpdoc = document_get_from_notebook_page_num(window, current_index);
	
	return tmpdoc;
}
/* right now it's not needed anymore  
   as it's in the *doc->modified field 
	In the other hand, each char you type
	the modified flag is set so we have a
	memory acces that can be avoided.
	Perhaps in the future, this func  
	will be used again so DON'T DELETE IT */ 
gboolean
document_get_modified( Tcssed_doc* doc )
{
	ScintillaObject * sci;
	gint modified;
	
	sci = (ScintillaObject *) sci_get_from_document(  doc );	
	modified = SSM (sci, SCI_GETMODIFY, 0, 0);
	if( modified == 0) return FALSE;
	else return TRUE;	
}

void
document_set_modified( Tcssed_doc* doc , gboolean modified)
{
	ScintillaObject * sci;
	
	if( modified ){
		doc->modified = TRUE;
	}else{
		sci = (ScintillaObject *) sci_get_from_document(  doc );	
		SSM (sci, SCI_SETSAVEPOINT, 0, 0);
		doc->modified = FALSE;
	}	
}
/*  Detach the doc from the notebook , pops it out the list
	and clear all memory allocated. If last doc in queue it
	must open a clean one.
	Save before it or changes will be lost.
*/
gboolean
document_dispose( Tcssed_doc* doc )
{
	GtkWidget* notebook;
	GtkWidget* editor;
	Tcssed_window* window;	
	GList* tmplist;
	gint document_page_number;
	gint notebook_total_pages;
	gboolean needs_new = FALSE;
	
	notebook = doc->notebook;
	editor = doc->editor;
	
	document_page_number = gtk_notebook_page_num(GTK_NOTEBOOK(notebook),editor);
	
	if( document_page_number == -1 ){ // editor don't exist
		return FALSE;
	}else{
		gtk_notebook_remove_page(GTK_NOTEBOOK(notebook), document_page_number);
#ifdef GTK_IS_2_2
		notebook_total_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK(notebook));
#else
		notebook_total_pages = g_list_length(GTK_NOTEBOOK(notebook)->children);
#endif
		if( notebook_total_pages == 0 ){
			needs_new = TRUE;
#ifdef DEBUG
			g_print("document.c:document_dispose() - Last notebook page out.\nWill create new one ...\n\n");
#endif	
		}	
		// clear doc allocated mem and pop out the doc list
		if( doc->filename != NULL ){
			g_free(doc->filename);
		}
		window = doc->window;
		tmplist = window->document_list;
		window->document_list = g_list_remove(tmplist,doc);
		g_free(doc);
	}

	if( needs_new )	create_and_attach_new_doc (window,"Untitled");
	return TRUE;
}

/* line numbers visibility */
gboolean 
document_get_line_numbers_visible( Tcssed_doc* doc )
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	return sci_get_line_numbers(sci);
}

void 
document_set_line_numbers_visible( Tcssed_doc* doc, gboolean visible )
{
	ScintillaObject* sci;	
	
#ifdef DEBUG
	g_print("Going to set line numbers visibility to %s in doc %x\n",visible?"true":"false",doc);	
#endif	
	sci = sci_get_from_document(doc);
	sci_set_line_numbers(sci,visible);
}
/* folding marging visibility */
gboolean 
document_get_folding_margin_visible( Tcssed_doc* doc )
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	return sci_get_folding_margin_visible(sci);
}

void 
document_set_folding_margin_visible( Tcssed_doc* doc, gboolean visible )
{
	ScintillaObject* sci;	
	
#ifdef DEBUG
	g_print("Going to set line numbers visibility to %s in doc %x\n",visible?"true":"false",doc);	
#endif	
	sci = sci_get_from_document(doc);
	sci_set_folding_margin_visible(sci,visible);
}
/* line endings visibility */
gboolean
document_get_line_endings_visible( Tcssed_doc* doc)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	return sci_get_visible_eols(sci);	
}
void
document_set_line_endings_visible( Tcssed_doc* doc, gboolean visible)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	sci_set_visible_eols(sci,visible);	
}
/* line wrapping */
gboolean
document_get_lines_wrapped( Tcssed_doc* doc)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	return sci_get_lines_wrapped(sci);	
}
void
document_set_lines_wrapped( Tcssed_doc* doc, gboolean wrapped)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	sci_set_lines_wrapped(sci,wrapped);	
}
/* white spaces */
gboolean
document_get_white_spaces_visible( Tcssed_doc* doc)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	return sci_get_visible_white_spaces(sci);	
}
void
document_set_white_spaces_visible( Tcssed_doc* doc, gboolean visible)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	sci_set_visible_white_spaces(sci,visible);	
}
/* EOL mode */
gint
document_get_eol_mode( Tcssed_doc* doc )
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);	
	return sci_get_eol_mode	(sci);
}

void 
document_set_eol_mode(Tcssed_doc* doc, gint eolmode)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);	
	sci_set_eol_mode(sci, eolmode);
}

void 
document_convert_eols(Tcssed_doc* doc, gint eolmode)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);	
	sci_convert_eols(sci, eolmode);
}
/* text */
void 
document_add_text( Tcssed_doc* doc, gchar* text )
{
	ScintillaObject* sci;	
	sci = sci_get_from_document( doc );
	sci_add_text(sci,text);
}

void/* adds text to the document editor ending in line feed */
document_add_text_line( Tcssed_doc * doc, char* text )
{
	ScintillaObject * sci;
	sci = SCINTILLA( doc->editor );

	SSM (sci, SCI_ADDTEXT, strlen(text) , (sptr_t) text );
	SSM (sci, SCI_ADDTEXT, strlen("\n") , (sptr_t) "\n" );
}
void // no wrapper function for this
document_search_text( Tcssed_doc* doc, gchar* text, gint flags )
{
	ScintillaObject* sci;	
	gint selection_start;
	gint selection_end;	
	gint search_pos;
#ifdef DEBUG	
	gint current_pos;
	gint anchor_pos;
#endif	
	
	sci = sci_get_from_document( doc );	
	// look if any selection, if so move the caret out it can
	// it can be selected by a prevous search.
	// right now, cssed can't search into a block of text selected by
	// the user. The user must know that.
	
	selection_start = SSM( sci, SCI_GETSELECTIONSTART, 0, 0);
	selection_end   = SSM( sci, SCI_GETSELECTIONEND, 0, 0);
	
#ifdef DEBUG	
	current_pos		= SSM( sci, SCI_GETCURRENTPOS, 0, 0);
	anchor_pos		= SSM( sci, SCI_GETANCHOR, 0, 0);
	g_print( "document_search_text(): Selection start %d end %d pos %d anchor %d\n",selection_start,selection_end,current_pos,anchor_pos);
#endif

	if( selection_end > selection_start ){ // there's selection
		// FIXME: build a wrapper function
		SSM( sci, SCI_GOTOPOS, selection_end + 1, 0);
	}	
	// set the anchor
	// FIXME: build a wrapper function
	SSM( sci, SCI_SEARCHANCHOR,0,0);
	// FIXME: build a wrapper function
	search_pos = SSM( sci, SCI_SEARCHNEXT,flags,(sptr_t) text );
	if( search_pos != -1 ){
		int line = document_get_line_from_position( doc, search_pos );
		if( !document_get_line_is_visible( doc, line ) ){
			document_ensure_line_is_visible( doc, line );	
		}
		// FIXME: build a wrapper function
		SSM( sci, SCI_SCROLLCARET,0,0);
	}else{		
		cssed_error_message
		(
		_("The text can't be found or end of document reached.\n\nIf you're \
sure the text exists,then the document is inactive.\nClick on a document to make it active"),
		_("End of document reached") 
		);
	}
}

void 
document_undo(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_undo(sci);
}
void 
document_redo(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_redo(sci);
}

void 
document_end_undo_action(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_end_undo_action(sci);
}

void 
document_start_undo_action (Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_start_undo_action(sci);
}

void 
document_new_undo_action(Tcssed_doc* doc)
{
	document_end_undo_action(doc);
	document_start_undo_action (doc);
}

void 
document_empty_undo_buffer(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_empty_undo_buffer(sci);
}

void 
document_set_undo_collection(Tcssed_doc* doc, gboolean set)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_set_undo_collection(sci,set);
}

gboolean
document_get_undo_collection(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_get_undo_collection(sci);
}

void 
document_zoom_in(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_zoom_in(sci);
}

void 
document_zoom_out(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_zoom_out(sci);
}

void 
document_zoom_off(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_zoom_off(sci);
}

void 
document_cut (Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	sci_cut (sci);
}
void 
document_copy (Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_copy (sci);
}
void 
document_paste (Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_paste (sci);
}

void 
document_set_marker_at_line( Tcssed_doc* doc, gint line, gboolean set, gint marker)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_set_marker_at_line( sci, line, set, marker);
}
gboolean
document_is_marker_set_at_line( Tcssed_doc* doc, gint line, gint marker)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	return sci_is_marker_set_at_line( sci, line, marker);
}
gboolean 
document_marker_next(Tcssed_doc* doc, gint line, gint marker_mask)
{
	ScintillaObject* sci;
	gint position;
	
	sci = sci_get_from_document( doc );	
	return sci_marker_next( sci, line, marker_mask );
}
	
	
gboolean 
document_marker_prev(Tcssed_doc* doc, gint line, gint marker_mask)
{
	ScintillaObject* sci;
	gint position;
	
	sci = sci_get_from_document( doc );		
	return sci_marker_prev( sci, line, marker_mask );
}

gint
document_get_line_from_position( Tcssed_doc* doc, gint position)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	return sci_get_line_from_position(sci, position );
}
gint
document_get_position_from_line( Tcssed_doc* doc, gint line)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	return sci_get_position_from_line(sci, line );
}
gint 
document_get_current_position	(Tcssed_doc* doc )
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );		
	return sci_get_current_position	(sci);
}

void
document_set_current_position	(Tcssed_doc* doc, gint position)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );		
	sci_set_current_position(sci, position);
}

void
document_set_current_line(Tcssed_doc* doc, gint line)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );		
	sci_set_current_line(sci, line);
}
gint
document_get_selection_start(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	sci_get_selection_start	(sci);
}

gint
document_get_selection_end(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	sci_get_selection_end(sci);
}

void
document_set_selection_start(Tcssed_doc* doc, gint position)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	sci_get_selection_start	(sci);
}

void
document_set_selection_end(Tcssed_doc* doc, gint position)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	sci_get_selection_end(sci);
}

gint 
document_get_line_end_from_position(Tcssed_doc* doc, gint position)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	return sci_get_line_end_from_position(sci, position);
}

void  // make a sci wrapper for this
document_set_selection_range(Tcssed_doc* doc,gint start, gint end)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	SSM(sci,SCI_SETSEL, end, start);
	//g_print( "Anchor pos %d current pos %d\n", end , start);
	//document_set_selection_start(doc,start);
	//document_set_selection_end(doc, end);
}

void
document_replace_sel(Tcssed_doc* doc, gchar* text)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	sci_replace_sel(sci, text);
}
void
document_clear_sel(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	sci_replace_sel(sci, "");
}

gint
document_get_length	(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	return sci_get_length(sci);	
}

void
document_get_text(Tcssed_doc* doc, gint len, gchar* text)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	return sci_get_text(sci,len,text);
}

void 
document_write_error_to_program_output(Tcssed_doc* doc, gchar* error_message,gchar* color_desc)
{
	Tcssed_window *window;

	GtkListStore *store;
	GtkTreeIter iter;
	GtkWidget* output_list;
	GtkTreePath* path;
	gchar* markup;

	
	window = CSSED_WINDOW(doc->window);
	store = window->output_store;
	
	
	markup = g_strdup_printf("<span color='%s'>%s</span>",color_desc,error_message);
	gtk_list_store_insert_before(store,&iter,NULL);
	gtk_list_store_set (store, &iter, 0, markup, -1);
	g_free( markup );
	
	output_list = lookup_widget(GTK_WIDGET(doc->notebook),"output_list");
	// switch to the fist tab, must be configurable as can be anoying
	gtk_notebook_set_current_page (GTK_NOTEBOOK(window->notebook_utils),0);

	path = gtk_tree_model_get_path (GTK_TREE_MODEL(store),&iter);
	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW(output_list),path,NULL,FALSE,0,0);
}
gboolean
document_current_pos_is_into_selector(Tcssed_doc* doc)
{
	ScintillaObject *sci;

	gint position;
	gint open_brace;
	gint close_brace;
	gchar *buffer;



	sci = (ScintillaObject *) sci_get_from_document (doc);
	position = document_get_current_position (doc);


	SSM (sci, SCI_SEARCHANCHOR, 0, 0);
	open_brace = SSM (sci, SCI_SEARCHPREV, 0, (sptr_t) "{");

#ifdef DEBUG
	if (open_brace == -1)
		g_print ("No open brace before\n");
	else
		g_print ("Open brace at %d\n", open_brace);
#endif

	SSM (sci, SCI_SEARCHANCHOR, 0, 0);
	close_brace = SSM (sci, SCI_SEARCHNEXT, 0, (sptr_t) "}");

#ifdef DEBUG
	if (close_brace == -1)
		g_print ("No close brace after\n");
	else
		g_print ("Close brace at %d\n", close_brace);
#endif

	if (close_brace < position)
	{
		document_set_current_position(doc,position);
		return FALSE;
	}
	else
	{
		if ((open_brace != -1) && (close_brace != -1))
		{	
			document_set_current_position(doc,position);
			return TRUE;
		}else{
			document_set_current_position(doc,position);
			return FALSE;
		}
	}	
}
gint 
document_get_line_length(Tcssed_doc* doc, gint line)
{
	ScintillaObject *sci;
	sci = sci_get_from_document (doc);	
	return sci_line_length(sci,line);
}

gint
document_get_line_count( Tcssed_doc* doc )
{
	ScintillaObject *sci;
	sci = sci_get_from_document (doc);		
	return sci_get_line_count( sci );
}

gint 
document_get_text_line(Tcssed_doc* doc, gint line,gchar* buffer)
{
	ScintillaObject *sci;

	sci = (ScintillaObject *) sci_get_from_document (doc);	
	sci_get_line(sci, line, buffer);
}
void  // not in document.h
document_get_xy_from_position(Tcssed_doc* doc,gint pos, gint* x, gint* y)
{
	ScintillaObject *sci;

	sci = (ScintillaObject *) sci_get_from_document (doc);	
	sci_get_xy_from_position(sci, pos, x, y);	
}
void   // not in document.h
document_gtk_popmenu_position_func(GtkMenu *menu, gint *x, gint *y,
				gboolean *push_in,  gpointer user_data)
{
	Tcssed_point* point;
	Tcssed_window* window;
	Tcssed_doc* doc;
	GtkWidget* notebook;
	GtkWidget* editor;
	GtkWidget* window_widget;
	GdkWindow* gdkwindow;
	gint new_x;
	gint new_y;
	gint root_x;
	gint root_y;
		
	point = (Tcssed_point*) user_data;
	window = CSSED_WINDOW(point->main_window);
	doc = document_get_current(window);
	notebook = (GtkWidget*) window->notebook_docs;
	editor = GTK_WIDGET(doc->editor);
	window_widget = gtk_widget_get_toplevel(notebook);
	gdkwindow = gtk_widget_get_parent_window (editor);	
#ifdef DEBUG
	g_print("document_gtk_popmenu_position_func:\n\tis notebook %d\n",GTK_IS_NOTEBOOK(notebook));
#endif
	gtk_widget_translate_coordinates        (editor,
                                             window_widget,
                                             point->x,
                                             point->y,
                                             &new_x,
                                             &new_y);											 
#ifdef DEBUG
	g_print("Translated coordinates from editor to window\n\tx = %d , y = %d\n",
			new_x, new_y);
#endif	
	gdk_window_get_root_origin(gdkwindow, &root_x, &root_y);
#ifdef DEBUG
	g_print("Root window coordinates\n\tx = %d , y = %d\n",
			root_x,root_y);
#endif	
	*x = new_x + root_x + 5;  // +5  add a little padding to right
	*y = new_y + root_y + 30; // +30 add a little padding down
#ifdef DEBUG
	g_print("Menu coordinates\n\tx = %d , y = %d\n",
			*x,*y);
#endif		
	g_free(user_data);
}
/* It will scan the second treeview colum ( in the css definitions treeview )
	trying to find the text passed. If it's found, it builds a popmenu with the
	third column contents and connect the appropiated callbacks to get 
	autocompletion to work */
void // not in document.h
document_pop_submenu_from_keyword_in_treeview(Tcssed_doc* doc,gchar* text)
{
	Tcssed_window* window;
	Tcssed_point* point;
	Tcssed_popmenu_data* popmenu_data;
	//GdkColor color;
	GtkWidget* treeview;
	GtkTreeModel* model;
	GtkWidget* menu;
	GtkWidget* menu_items;
	GtkTreeIter iter;
	gboolean valid;
	gboolean validchild;
	gchar* keyword;	
	gchar* value;
	gchar* path_str;
	GList* menu_item_list = NULL;
	gint nrow;
	gint x;
	gint y;
	
	window = CSSED_WINDOW(doc->window);
	treeview = window->tree_view_css_properties;
	
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
	
	if ( model == NULL ){
		return;
	}
	valid = gtk_tree_model_get_iter_from_string(model,&iter,"0:0");
	if(!valid){
		return;
	}
	nrow = 0;
	while(valid){
		gtk_tree_model_get (model, &iter, 1, &keyword, -1);
#ifdef DEBUG
        g_print ("scanning %s\n", keyword);
#endif
		if( strcmp(keyword,text)==0 ){
#ifdef DEBUG			
			g_print("found !!\n");
#endif			
			path_str = g_strdup_printf("0:%d:0",nrow);
			valid =  gtk_tree_model_get_iter_from_string(model,&iter,path_str);
#ifdef DEBUG			
			g_print("trying to get child on %s\n",path_str);
#endif			
			g_free(path_str);
			if(!valid ){
#ifdef DEBUG				
				g_print("can't get child.\n");
#endif					
				g_free(keyword);
				return;
			}
#ifdef DEBUG			
			g_print("got 3rd child!\n");
#endif				
			menu = gtk_menu_new ();
	
			while( valid ){	
				popmenu_data = g_malloc(sizeof(	Tcssed_popmenu_data ));
				gtk_tree_model_get (model, &iter, 2, &value, -1);
				menu_items = gtk_menu_item_new_with_label (value);
#ifdef DEBUG				
				g_print("\t* %s\n",value);
#endif					
				popmenu_data->main_window = window;
				popmenu_data->document = document_get_current(window);
				popmenu_data->attribute = g_strdup(value);
				popmenu_data->property = g_strdup(keyword);
				popmenu_data->add_property_string = FALSE;
				
				menu_item_list = g_list_append(menu_item_list, popmenu_data);
				gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_items);
				
				// callback will call fire_dialog_from_strings_or_write_output()
				g_signal_connect (G_OBJECT (menu_items), "activate",
                                      G_CALLBACK (popmenu_autocomp_activate_cb), 
                                      popmenu_data );				
								  
				gtk_widget_show (menu_items);
				g_free(value);
				valid = gtk_tree_model_iter_next (model,&iter);				
			}
			
			// this will dispose all memory allocated parsing the list 
			// passed on unmap event (event->type ==  GDK_UNMAP).
			g_signal_connect (G_OBJECT (menu), "event-after",
                               G_CALLBACK (popmenu_autocomp_event_after_cb), 
                               menu_item_list );			
#ifdef DEBUG
			g_print("Sending list %p as user_data for callback\n",menu_item_list);
#endif		
			document_get_xy_from_position(doc,document_get_current_position(doc),&x,&y);
#ifdef DEBUG	
			g_print("Current point into editor x=%d,y=%d\n",x,y);
#endif				
			point = g_malloc(sizeof(Tcssed_point));
			point->main_window = window;
			point->x = x;
			point->y = y;
			gtk_menu_popup (GTK_MENU(menu),NULL, NULL,
										 document_gtk_popmenu_position_func  ,
                                         (gpointer) point,// freed by callback
                                         0,
                                         gtk_get_current_event_time());
			g_free(keyword);
			return;				
		}
		nrow++;
        g_free (keyword);
		valid = gtk_tree_model_iter_next (model,&iter);
     }
}

void 
document_show_autocompletion_list(Tcssed_doc* doc, gint identifier, gchar* list)
{
	ScintillaObject* sci;
	sci = sci_get_from_document( doc );	
	SSM(sci,SCI_AUTOCSHOW,0, (sptr_t) list); 
}
/* folding */

void
document_toggle_fold_at_line(Tcssed_doc* doc, gint line)
{
	ScintillaObject* sci;
	sci = sci_get_from_document( doc );	
	sci_toggle_fold(sci,line);
}

gboolean
document_get_line_is_visible(Tcssed_doc* doc, gint line)
{
	ScintillaObject* sci;
	sci = sci_get_from_document( doc );	
	return sci_get_line_is_visible(sci,line);	
}

void
document_ensure_line_is_visible(Tcssed_doc* doc, gint line)
{
	ScintillaObject* sci;
	sci = sci_get_from_document( doc );		
	sci_ensure_line_is_visible( sci, line);
}

gint
document_get_fold_level(Tcssed_doc* doc, gint line)
{
	ScintillaObject* sci;
	sci = sci_get_from_document( doc );		
	return sci_get_fold_level(sci,line);
}

void
document_unfold_all( Tcssed_doc* doc )
{
	gint lines;
	gint pos;
	int i;
	lines = document_get_line_count( doc );
	pos = document_get_current_position( doc );
	for( i=0; i<lines;i++){
		document_ensure_line_is_visible( doc, i );
	}
	document_set_current_position( doc,  0 );
}

void
document_fold_all( Tcssed_doc* doc )
{
	gint lines;
	gint pos;
	int i;
	
	if(!document_get_folding_enabled( doc ) ){
		document_set_folding_margin_visible( doc ,TRUE );	
	}
	
	lines = document_get_line_count( doc );
	pos = document_get_current_position( doc );
	for( i=0; i<lines;i++){
		int level =document_get_fold_level(doc,i);
		if (level & SC_FOLDLEVELHEADERFLAG) {
			if( document_get_fold_expanded_at_line(doc, i) ){
				document_toggle_fold_at_line(doc, i);
			}
		}
	}
	document_set_current_position( doc,  0 );
}
gboolean 
document_get_fold_expanded_at_line( Tcssed_doc* doc, gint line)
{
	ScintillaObject* sci;
	sci = sci_get_from_document( doc );		
	return sci_get_fold_expanded(sci, line);
}

gboolean
document_get_folding_enabled( Tcssed_doc* doc )
{
	return document_get_folding_margin_visible( doc );
}
/*  FIXME
	This and the function run_dialog_disconnected_from_string in 
	floating-list.c must change to a hash table to be quicker than this.
	Now that the changes are done, we must build and structure 
	cssdialogs_class with pointers to:
		* the dialog trigger keyword
		* the dialog contruction function
		* the dialog "get:string" function in cssdialogs_functions.c
		* the destruction function (if not any special gtk:widget_destroy)
	Then we can use the same structure for both functions.
	The structures will be in a doubly linked list in the window
	structure.
	If finally a hash table, the key should be the dialog trigger 
	keyword.
*/
void 
fire_dialog_from_strings_or_write_output(Tcssed_window* window, 
										Tcssed_doc* tmpdoc, 
										gchar* attribute, 
										gchar* property , 
										gboolean add_previous)
{
	gchar *line_to_insert;
	gchar* strdialog;
	GtkWidget *dialog;

	// FIXME use a hash table or something quicker than this
	if( attribute[0] == '(' ){ // if it don't start with ( is not a dialog
		if (strcmp (attribute, "(color)") == 0)
		{
			dialog = create_color_selector_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(angle)") == 0)
		{
			dialog = create_angle_property_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(background)") == 0)
		{
			dialog = create_background_property_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(border)") == 0)
		{
			dialog = create_border_properties_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(border-width)") == 0)
		{
			dialog = create_border_width_properties_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(border-width-single)") == 0)
		{
			dialog = create_border_width_single_properties_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(increment-reset)") == 0)
		{
			dialog = create_counter_increment_reset_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(string)") == 0)
		{
			dialog = create_string_properties_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(uri)") == 0)
		{
			dialog = create_uri_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(attr)") == 0)
		{
			dialog = create_attr_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(counter)") == 0)
		{
			dialog = create_counter_property_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(font)") == 0)
		{
			dialog = create_font_property_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(font-family)") == 0)
		{
			dialog = create_font_family_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(font-size)") == 0)
		{
			dialog = create_font_size_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(frequency)") == 0)
		{
			dialog = create_frecuency_property_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(identifier)") == 0)
		{
			dialog = create_identifier_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(integer)") == 0)
		{
			dialog = create_integer_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(length)") == 0)
		{
			dialog = create_length_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(list-style)") == 0)
		{
			dialog = create_list_style_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(margin)") == 0)
		{
			dialog = create_margin_property_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(margin-width)") == 0)
		{
			dialog = create_margin_width_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}	
		else if (strcmp (attribute, "(number)") == 0)
		{
			dialog = create_number_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);
		}	
		else if (strcmp (attribute, "(padding)") == 0)
		{
			dialog = create_padding_property_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(outline)") == 0)
		{
			dialog = create_outline_property_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(pause)") == 0)
		{
			dialog = create_pause_property_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(percentage)") == 0)
		{
			dialog = create_percentage_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);			
		}			
		else if (strcmp (attribute, "(percentage-length)") == 0)
		{
			dialog = create_percentage_length_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);			
		}		
		else if (strcmp (attribute, "(play-during)") == 0)
		{
			dialog = create_play_during_property_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(quotes)") == 0)
		{
			dialog = create_quotes_property_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(shape)") == 0)
		{
			dialog = create_shape_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(size-length)") == 0)
		{
			dialog = create_size_length_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(text-shadow)") == 0)
		{
			dialog = create_text_shadow_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(voice-family)") == 0)
		{
			dialog = create_voice_family_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(width-style-color)") == 0)
		{
			dialog = create_border_width_style_color_value_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);			
		}
		else if (strcmp (attribute, "(border-style)") == 0)
		{
			dialog = create_border_style_dialog (window,
												g_strdup (property) ,
												add_previous,
												TRUE);
			gtk_widget_show (dialog);			
		}		
		else
		{ // dialog don't exists yet
			strdialog = g_strdup_printf(_("The dialog %s doesn't exist."),attribute);
			document_write_error_to_program_output(tmpdoc,strdialog,"black");
			document_new_undo_action(tmpdoc);
			if( add_previous ){
				line_to_insert =
				g_strconcat (property, ": ", attribute, ";",
					     	NULL);
				document_add_text_line (tmpdoc, line_to_insert);
				document_add_text (tmpdoc, "\t");
				g_free (line_to_insert);
			}else{
				document_add_text (tmpdoc, attribute);
			}
			document_new_undo_action(tmpdoc);
			g_free(strdialog);			
		}		
	}
	else
	{// no dialog to show, write the selected strings
		if (tmpdoc != NULL)
		{
			document_new_undo_action(tmpdoc);
			if( add_previous ){
				line_to_insert =
					g_strconcat (property, ":", attribute, ";",
					     NULL);
				document_add_text_line (tmpdoc, line_to_insert);
				document_add_text (tmpdoc, "\t");
				g_free (line_to_insert);
			}else{
				document_add_text (tmpdoc, attribute);
			}
			document_new_undo_action(tmpdoc);
		}
	}
}

// callback func called by all editors when a signals arises
void
on_editor_notification( GtkWidget* editor,gint scn, gpointer lscn, gpointer user_data)
{
	struct SCNotification *nt;
	ScintillaObject* sci;
	Tcssed_doc* tmpdoc;
	Tcssed_window* window;
	// used if lastchar == ':' in Autocompletion mode
	gint linestart;
	gchar* buffer;
	gint line;
	gint linelength;
	gchar lastchar;
	gint position;
	gint margin;
	
	window = ( Tcssed_window*) user_data;
	tmpdoc = document_get_from_editor( window, editor );

	nt = lscn;
	switch (nt->nmhdr.code){
		case SCN_STYLENEEDED:	
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %p notification SCN_STYLENEEDED\n", editor);
#endif
			break;
		case SCN_CHARADDED:	
			 lastchar = nt->ch;
 			// set undo when space or new line is typed by the user
			// ( no entered by any function as document_add_text )
			if( (lastchar == ' ') || (lastchar == '\n') || (lastchar == '\t') ){
				document_new_undo_action(tmpdoc);
			}
 
			if( tmpdoc->autocompletion ){
				// add close brace if open brace is added
				if( lastchar == '{' ){
					document_add_text(tmpdoc,"\n\t\n}\n");
					document_set_current_position(tmpdoc,
						(document_get_current_position(tmpdoc)-3) );
					document_show_autocompletion_list(tmpdoc,0,(gchar*) window->keywords_list);
				}
				if( lastchar == ';'){
					document_add_text(tmpdoc,"\n\t");		
					document_show_autocompletion_list(tmpdoc,0,(gchar*) window->keywords_list);
				}			
	
				if( lastchar == ':'){
					// get if in selector
					if(document_current_pos_is_into_selector(tmpdoc)){
#ifdef DEBUG
						g_print("into a selector\n");
#endif				
						position = document_get_current_position(tmpdoc);
						line = document_get_line_from_position(tmpdoc,position);
						linestart  = document_get_position_from_line(tmpdoc,line);
						
						// we can not assume the user is at the end of a line 
						// so check if we are. reserve memory for the whole line
						linelength = document_get_line_length(tmpdoc,line);						
#ifdef DEBUG					
						g_print("Line %d start %d pos %d length %d\n",line,linestart,position,linelength);
#endif					
						if( position > linestart){	
							buffer = g_malloc0(sizeof(gchar)*( linelength )+1);					
							document_get_text_line(tmpdoc,line,buffer);
#ifdef DEBUG							
							g_print("Line contains: \"%s\"\n", buffer);	
#endif							
							buffer[position-linestart-1] = '\0';
							g_strstrip( buffer );
#ifdef DEBUG						
							g_print("Cleaned: \"%s\"\n", buffer);
#endif							
							document_pop_submenu_from_keyword_in_treeview(tmpdoc,buffer);						
							g_free(buffer);	
						}	
					}
				}	
			}
#ifdef DEBUG_CONTROL_NOTIFICATIONS 			
								g_print("received control %p notification SCN_CHARADDED\n", editor);
#endif
			break;
		case SCN_SAVEPOINTREACHED:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %p notification SCN_SAVEPOINTREACHED\n", editor);
#endif
			break;
		case SCN_SAVEPOINTLEFT:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %p notification SCN_SAVEPOINTLEFT\n", editor);
#endif
			break;
		case SCN_MODIFYATTEMPTRO:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %p notification SCN_MODIFYATTEMPTRO\n", editor);
#endif
			break;
		case SCN_KEY:
			if( (nt->ch=='a') && (nt->modifiers==4) ){ // Alt + A fire autocompletion
				document_show_autocompletion_list(tmpdoc,0,(gchar*) window->keywords_list);
			}else if( (nt->ch=='l') && (nt->modifiers==4) ){ // Alt + l selects current line
				gint kill_line;
				gint start_pos;
				gint line_len;
				gint end_pos;
				gint curpos;
				
				curpos = document_get_current_position(tmpdoc);
				kill_line = document_get_line_from_position(tmpdoc, curpos);
				start_pos = document_get_position_from_line(tmpdoc, kill_line);
				line_len = document_get_line_length(tmpdoc, kill_line);				
				end_pos = start_pos + line_len;								
				document_set_selection_range(tmpdoc, start_pos, end_pos);
			}else if((nt->ch==';') && ((nt->modifiers==5)||(nt->modifiers==4))){ // Alt + ";" adds ; (no auto complete)
				document_add_text(tmpdoc,";");
			}else if( (nt->ch=='s') && (nt->modifiers==4) ){ // Alt + s scan selector
				on_menu_scan_selector_activate (NULL, window);
			}
#ifdef DEBUG
			g_print("Pressed char %d modifier %d\n", nt->ch,nt->modifiers);
#endif
			break;
		case SCN_MODIFIED:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
			g_print("received control %p notification SCN_MODIFIED\n", editor);
#endif
			if( tmpdoc != NULL ){
				tmpdoc->modified = TRUE;
			}else{
				g_print("Program error, can set the dirty flag on document\nTo cssed it will seems that never changes !!!\n");
			}		
			break;
		case SCN_MACRORECORD:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 						
		g_print("received control %p notification SCN_MACRORECORD\n", editor);
#endif
			break;
		case SCN_MARGINCLICK:
			position = nt->position;
			margin = nt->margin;
#ifdef DEBUG		
			g_print("Clicked margin %d\n",margin);
#endif		
			line = document_get_line_from_position( tmpdoc, position );
			if( margin == 1){
				if( document_is_marker_set_at_line( tmpdoc, line, 0 ) ){
					document_set_marker_at_line(tmpdoc,line,TRUE, 0 );
				}else{
					document_set_marker_at_line(tmpdoc,line,FALSE, 0 );
				}
			}else if( margin == 2 ){
				int level =document_get_fold_level(tmpdoc,line);
				if (level & SC_FOLDLEVELHEADERFLAG) {
#ifdef DEBUG					
					g_print("Fold toggled at line %d\n",line);
#endif						
					document_toggle_fold_at_line(tmpdoc, line);
                }
 			}
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
			g_print("received control %p notification SCN_MARGINCLICK\n", editor);
#endif
			break;
		case SCN_USERLISTSELECTION:
			 /* only one user list in
			switch( nt->wParam ){
				case 0:{
					g_print("Selected from list 0: %s\n",(gchar*) nt->text);
					}
				break;
				default:
						g_print("Unknow list selected\n");
				break;				
			}*/
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %p notification SCN_USERLISTSELECTION\n", editor);
#endif
			break;
		default:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received spurious control %p notification %d\n", editor, nt->nmhdr.code );
#endif
			break;
	}

	return;
}
/*






*/

/* scintilla object related functions */
ScintillaObject*
sci_get_from_notebook_page(GtkNotebook* notebook, gint page_number)
{
	ScintillaObject* sci;
	GtkWidget* editor;
	
	editor = gtk_notebook_get_nth_page (notebook, page_number);
	sci = SCINTILLA(editor);
	return sci;	
}

ScintillaObject*
sci_get_from_document( Tcssed_doc* doc )
{
	ScintillaObject * sci;
	sci = SCINTILLA( doc->editor );	
	return sci;
}

/* it's unused right now see document_set_css_style*/
void
sci_set_css_style (ScintillaObject * sci)
{
	gint cred = 0x00ff00;
	gint colive = 0x004400; //
	gint cyellow = 0x009999;
	gint cblue = 0x993300;
	gint cnavy = 0x990000;
	gint cblack = 0x000000;
	
	SSM (sci, SCI_STYLECLEARALL, 0, 0);
	SSM (sci, SCI_SETLEXER, SCLEX_CSS, 0);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_DEFAULT, 0x000077); 
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_TAG, 0xA46621 ); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_TAG, 1);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_PSEUDOCLASS, 0x990066);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_UNKNOWN_PSEUDOCLASS, 0x0000ff); 
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_OPERATOR, 0x000099); 
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_IDENTIFIER, 0x990000); 
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_IDENTIFIER, 1);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_UNKNOWN_IDENTIFIER , 0x0000ff);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_VALUE, 0x222222); 
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_COMMENT, 0x666666);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_IMPORTANT, 0x000099);
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_IMPORTANT, 1);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_DIRECTIVE, 0x660066);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_DOUBLESTRING, 0x660033);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_SINGLESTRING, 0x660033);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_CLASS, colive);
	SSM (sci, SCI_STYLESETBOLD,	SCE_CSS_CLASS, 1);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_ID, colive); // 0x009999);
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_ID, 1);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_VALID_VALUE, 0x003399);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_FUNCTION,  0x993300);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_NUMBER, 0x330099);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_UNIT, 0x990000);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_COLOR, 0x006600);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_HEXACOLOR, 0x006600);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_ATTR_MATCH, 0x004444);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_LANGUAGE, cnavy);
	SSM (sci, SCI_SETTABWIDTH, 4, 0);
	SSM (sci, SCI_SETREADONLY, 0, 0);
	SSM (sci, SCI_SETMARGINTYPEN, 0, SC_MARGIN_NUMBER);
	SSM (sci, SCI_SETMARGINWIDTHN, 0, 
				SSM(sci, SCI_TEXTWIDTH, STYLE_LINENUMBER,
				(sptr_t) MARGIN_LINES_WIDTH_STRING));
	SSM (sci, SCI_SETMARGINTYPEN, 1, SC_MARGIN_SYMBOL);
	SSM (sci, SCI_SETMARGINWIDTHN, 1, MARGIN_MARKERS_WIDTH_PIXELS );
	SSM (sci, SCI_SETMARGINSENSITIVEN,1, TRUE);
	SSM (sci, SCI_MARKERDEFINE,0, SC_MARK_ROUNDRECT); // marker 0 for bookmarks	
	SSM (sci,SCI_SETUNDOCOLLECTION,1,0);
	SSM (sci,SCI_SETMODEVENTMASK, SC_MODEVENTMASKALL ,0); 
	SSM (sci, SCI_SETPROPERTY, (sptr_t) "fold", (sptr_t) "1");
	SSM (sci, SCI_SETPROPERTY, (sptr_t) "fold.compact", (sptr_t) "1");
 	SSM (sci, SCI_SETFOLDFLAGS, 16,0);
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDEROPEN, SC_MARK_BOXMINUS);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDEROPEN, 0xffffff);
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDEROPEN, 0x000000);
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDER, SC_MARK_BOXPLUS);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDER, 0xffffff);
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDER, 0x000000);	
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDERSUB, SC_MARK_VLINE);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDERSUB, 0xffffff);
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDERSUB, 0x000000);	
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDERTAIL, SC_MARK_LCORNER);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDERTAIL, 0xffffff);	
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDERTAIL, 0x000000);	
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDEREND, SC_MARK_BOXPLUSCONNECTED);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDEREND, 0xffffff);
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDEREND, 0x000000);	
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDEROPENMID, SC_MARK_BOXMINUSCONNECTED);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDEROPENMID, 0xffffff);
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDEROPENMID, 0x000000);	
    SSM (sci,SCI_MARKERDEFINE,  SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_TCORNER);
	SSM (sci,SCI_MARKERSETFORE, SC_MARKNUM_FOLDERMIDTAIL, 0xffffff);
	SSM (sci,SCI_MARKERSETBACK, SC_MARKNUM_FOLDERMIDTAIL, 0x000000);	
	SSM (sci, SCI_SETMARGINSENSITIVEN,2, TRUE);
	SSM (sci, SCI_SETMARGINMASKN,2, SC_MASK_FOLDERS);
	SSM (sci, SCI_SETMARGINWIDTHN, 2, 0 );
}


void
sci_set_keyworkds (ScintillaObject * sci, gchar* keywords)
{
	enum {
		SC_KEYS_PROPERTIES,
		SC_KEYS_PSEUDO,
		SC_KEYS_VALUES,
		SC_KEYS_FUNCS,
		SC_KEYS_UNITS,
		SC_KEYS_COLORS
	};
	
	const gchar* values = "above absolute always armenian auto avoid baseline behind below bidi-override \
blink block bold bolder both bottom capitalize caption center center-left center-right child circle cjk-ideographic close-quote \
code collapse collapse compact condensed continuous crop cross crosshair dashed decimal decimal-leading-zero default digits \
disc dotted double embed e-resize expanded extra-condensed extra-expanded far-left far-right fast female fixed \
georgian groove hebrew help hide high hiragana hidden higher hiragana-iroha icon inherit inline inset inside invert inline-table faster \
invert italic justify katakana katakana-iroha large larger landscape left left-side leftwards level lighter line-through list-item loud \
low lower lowercase lower-greek lower-alpha lower-latin lower-roman ltr male marker medium menu message-box middle mix move ne-resize \
no-close-quote none no-open-quote no-repeat normal nowrap n-resize nw-resize oblique once open-quote outset outside overline \
pointer portrait pre relative repeat repeat-x repeat-y ridge right rightwards right-side rtl run-in scroll semi-condensed semi-expanded \
separate se-resize show silent slow slower small smaller small-caps small-caption side soft solid spell-out square s-resize static status-bar sub super sw-resize \
table table-caption table-cell table-column table-column-group table-footer-group table-header-group table-row table-row-group text thick thin \
top text-bottom text-top transparent ultra-condensed ultra-expanded underline upper-alpha uppercase upper-latin upper-roman visible wait wider w-resize x-fast x-high x-loud x-low \
x-slow x-soft  xx-small x-large xx-small x-small xx-large serif sans-serif cursive fantasy monospace";

	const gchar* namedColors = "aqua black blue fuchsia gray green lime maroon  navy olive purple red silver teal white yellow activeborder \
activecaption appworkspace background buttonface buttonhighlight buttonshadow buttontext captiontext graytext highlight highlighttext \
inactiveborder inactivecaption inactivecaptiontext infobackground infotext menu menutext scrollbar threeddarkshadow threedface threedhighlight \
threedlightshadow threedshadow window windowframe windowtext";
	
	SSM (sci, SCI_SETKEYWORDS, SC_KEYS_PROPERTIES, (sptr_t) keywords);
	SSM (sci, SCI_SETKEYWORDS, SC_KEYS_PSEUDO, (sptr_t) "link hover active visited first-letter first-line after before print visual aural first left first-child");
	SSM (sci, SCI_SETKEYWORDS, SC_KEYS_VALUES, (sptr_t) values);
	SSM (sci, SCI_SETKEYWORDS, SC_KEYS_FUNCS, (sptr_t) "counter attr url rgb rect");
	SSM (sci, SCI_SETKEYWORDS, SC_KEYS_UNITS, (sptr_t) "em ex px in cm mm pt pc \% mhz hz deg rad gra s ms");
	SSM (sci, SCI_SETKEYWORDS, SC_KEYS_COLORS, (sptr_t) namedColors);
}
/* line numbers visibility */
void 
sci_set_line_numbers(ScintillaObject * sci, gboolean set )
{
	if( set ){
			SSM (sci, SCI_SETMARGINWIDTHN, 0, 
				SSM(sci, SCI_TEXTWIDTH, STYLE_LINENUMBER,
				(sptr_t) MARGIN_LINES_WIDTH_STRING));	
	}else{
		SSM (sci, SCI_SETMARGINWIDTHN, 0, 0 );
	}
}
gboolean
sci_get_line_numbers(ScintillaObject * sci)
{
	gint margin_width;
	
	margin_width = SSM(sci, SCI_GETMARGINWIDTHN, 0, 0);
	if( margin_width > 0 ) return TRUE;
	else return FALSE;
}
/* folding margin visibility */
void 
sci_set_folding_margin_visible (ScintillaObject * sci, gboolean set )
{
	if( set ){
		SSM (sci, SCI_SETMARGINWIDTHN, 2, MARGIN_FOLDING_WIDTH_PIXELS );		
	}else{
		SSM (sci, SCI_SETMARGINWIDTHN, 2, 0 );
	}
}
gboolean
sci_get_folding_margin_visible(ScintillaObject * sci)
{
	gint margin_width;
	
	margin_width = SSM(sci, SCI_GETMARGINWIDTHN, 2, 0);
	if( margin_width > 0 ) return TRUE;
	else return FALSE;
}
/* end of lines */
void
sci_set_visible_eols(ScintillaObject* sci, gboolean set )
{
	SSM(sci,SCI_SETVIEWEOL,set,0);
}

gboolean 
sci_get_visible_eols(ScintillaObject* sci)
{
	return SSM( sci, SCI_GETVIEWEOL,0,0);
}
void
sci_set_visible_white_spaces(ScintillaObject* sci, gboolean set )
{
	if(set){
		SSM(sci,SCI_SETVIEWWS,SCWS_VISIBLEALWAYS,0);
	}else{
		SSM(sci,SCI_SETVIEWWS,SCWS_INVISIBLE,0);
	}
}
gboolean 
sci_get_visible_white_spaces(ScintillaObject* sci)
{
	return SSM( sci, SCI_GETVIEWWS,0,0);
}
void
sci_set_lines_wrapped(ScintillaObject* sci, gboolean set )
{
	if( set ){
		SSM(sci,SCI_SETWRAPMODE,SC_WRAP_WORD,0);
	}else{
		SSM(sci,SCI_SETWRAPMODE,SC_WRAP_NONE,0);
	}
}
gboolean 
sci_get_lines_wrapped(ScintillaObject* sci)
{
	return SSM( sci, SCI_GETWRAPMODE,0,0);
}
void 
sci_set_eol_mode( ScintillaObject* sci, gint eolmode)
{
	SSM( sci, SCI_SETEOLMODE, eolmode, 0);
}

gint
sci_get_eol_mode( ScintillaObject* sci)
{
	return SSM(sci,SCI_GETEOLMODE,0,0);
}

void 
sci_convert_eols( ScintillaObject* sci, gint eolmode)
{
	SSM( sci, SCI_CONVERTEOLS, eolmode,0);
}


void 
sci_add_text(ScintillaObject* sci, gchar* text)
{
	if( text != NULL ){// if null text is passed to scintilla will segfault
		SSM( sci, SCI_ADDTEXT, strlen(text) , (sptr_t) text);
	}
}

void
sci_undo( ScintillaObject* sci )
{
	if( SSM( sci, SCI_CANUNDO,0,0) ){
		SSM( sci, SCI_UNDO,0,0);		
	}else{ // change it to a document function
		cssed_error_message("There is no action in the undo list.","Undo limit reached");
	}	
}

void
sci_redo( ScintillaObject* sci )
{
	if( SSM( sci, SCI_CANREDO,0,0) ){
		SSM( sci, SCI_REDO,0,0);
	}else{// change it to a document function
		cssed_error_message("There is no action in the redo list.","Redo limit reached");
	}
}

void 
sci_start_undo_action( ScintillaObject* sci )
{
	SSM( sci,SCI_BEGINUNDOACTION,0,0 );
}

void 
sci_end_undo_action( ScintillaObject* sci )
{
	SSM( sci, SCI_ENDUNDOACTION,0,0);
}
void 
sci_set_undo_collection( ScintillaObject* sci, gboolean set )
{
	SSM( sci, SCI_SETUNDOCOLLECTION,set,0);
}

gboolean
sci_get_undo_collection( ScintillaObject* sci )
{
	return SSM( sci, SCI_GETUNDOCOLLECTION,0,0);
}

void 
sci_empty_undo_buffer( ScintillaObject* sci )
{
	SSM( sci, SCI_EMPTYUNDOBUFFER,0,0);
}

void 
sci_zoom_in( ScintillaObject* sci )
{
	SSM( sci, SCI_ZOOMIN,0,0);
}

void 
sci_zoom_out( ScintillaObject* sci )
{
	SSM( sci, SCI_ZOOMOUT,0,0);
}

void 
sci_zoom_off( ScintillaObject* sci )
{
	SSM( sci, SCI_SETZOOM,0,0);
}
void
sci_set_marker_at_line( ScintillaObject* sci, gint line_number, gboolean set, gint marker )
{
	
#ifdef DEBUG
	g_print("Set marker to %d line %d \n",set,line_number);
#endif
	
	if( set ){
		SSM( sci, SCI_MARKERADD,line_number, marker);
	}else{
		SSM( sci, SCI_MARKERDELETE,line_number, marker);
	}
}
gboolean 
sci_is_marker_set_at_line(ScintillaObject* sci, gint line, gint marker)
{
	gint state;
	
	state = SSM( sci, SCI_MARKERGET, line, marker );	
	return(!(state & (1 << marker)));
}

gboolean
sci_marker_next(ScintillaObject* sci, gint line, gint marker_mask)
{
	gint marker_line;

	marker_line = SSM(sci, SCI_MARKERNEXT, line, marker_mask);
#ifdef DEBUG	
	g_print( "Marker next: line %d next marker %d\n",line,marker_line);
#endif
	if( marker_line != -1 ){
		SSM(sci,SCI_GOTOLINE,marker_line,0);
		return TRUE;
	}else{
		return FALSE;
	}
}

gboolean
sci_marker_prev(ScintillaObject* sci, gint line, gint marker_mask)
{
	gint marker_line;
	
	marker_line = SSM(sci, SCI_MARKERPREVIOUS, line, marker_mask);	
#ifdef DEBUG		
	g_print( "Marker previous: line %d marker %d\n",line,marker_line);
#endif	
	if( marker_line != -1 ){
		SSM(sci,SCI_GOTOLINE,marker_line,0);
		return TRUE;
	}else{
		return FALSE;
	}
}

gint 
sci_get_line_from_position(ScintillaObject* sci, gint position )
{
	return SSM(sci, SCI_LINEFROMPOSITION, position, 0);
}

gint 
sci_get_position_from_line(ScintillaObject* sci, gint line )
{
	return SSM(sci, SCI_POSITIONFROMLINE, line, 0);
}
 
gint 
sci_get_current_position(ScintillaObject* sci )
{
	return SSM(sci, SCI_GETCURRENTPOS, 0, 0);
}

void 
sci_set_current_position(ScintillaObject* sci, gint position )
{
	SSM(sci, SCI_GOTOPOS, position, 0);
}

void 
sci_set_current_line(ScintillaObject* sci, gint line )
{
	SSM(sci, SCI_GOTOLINE, line, 0);
}

gint
sci_get_line_count( ScintillaObject* sci )
{
	return SSM(sci, SCI_GETLINECOUNT, 0, 0);	
}

void 
sci_set_selection_start(ScintillaObject* sci, gint position)
{
	SSM(sci, SCI_SETSELECTIONSTART, position, 0);	
}

void 
sci_set_selection_end(ScintillaObject* sci, gint position)
{
	SSM(sci, SCI_SETSELECTIONEND, position, 0);	
}

gint
sci_get_line_end_from_position(ScintillaObject* sci, gint position)
{
	SSM(sci, SCI_GETLINEENDPOSITION, position, 0);
}

void 
sci_cut(ScintillaObject* sci)
{
	SSM(sci, SCI_CUT, 0, 0);
}

void 
sci_copy(ScintillaObject* sci)
{
	SSM(sci, SCI_COPY, 0, 0);
}
void 
sci_paste(ScintillaObject* sci)
{
	SSM(sci, SCI_PASTE, 0, 0);
}
gint
sci_get_selection_start(ScintillaObject* sci)
{
	return SSM(sci, SCI_GETSELECTIONSTART,0,0);
}

gint
sci_get_selection_end(ScintillaObject* sci)
{
	return SSM(sci, SCI_GETSELECTIONEND,0,0);
}

void 
sci_replace_sel(ScintillaObject* sci, gchar* text)
{
	SSM(sci, SCI_REPLACESEL,0, (sptr_t) text);
}

gint
sci_get_length(ScintillaObject* sci)
{
	return SSM(sci,SCI_GETLENGTH,0,0);
}
gint 
sci_line_length(ScintillaObject* sci,gint line)
{
	return SSM(sci,SCI_LINELENGTH, line,0);
}
void // will not be null terminated
sci_get_line(ScintillaObject* sci, gint line, gchar* text)
{
	SSM(sci,SCI_GETLINE,line, (sptr_t) text);
}
void // the last char will be null terminated
sci_get_text(ScintillaObject* sci, gint len, gchar* text)
{
	SSM( sci, SCI_GETTEXT, len,(sptr_t) text );
}
void 
sci_get_xy_from_position(ScintillaObject* sci,gint pos, gint* x, gint* y)
{
	*x = SSM(sci, SCI_POINTXFROMPOSITION,0, (int) pos);
 	*y = SSM(sci, SCI_POINTYFROMPOSITION,0, (int) pos);
}
/* folding */
gboolean
sci_get_line_is_visible(ScintillaObject* sci, gint line)
{
	return SSM(sci,SCI_GETLINEVISIBLE, line,0); 
}
void
sci_ensure_line_is_visible( ScintillaObject* sci, gint line)
{
	 SSM(sci,SCI_ENSUREVISIBLEENFORCEPOLICY,line,0);
}

gint
sci_get_fold_level(ScintillaObject* sci, gint line)
{
	return SSM(sci,SCI_GETFOLDLEVEL, line,0);
}

void
sci_toggle_fold(ScintillaObject* sci, gint line)
{
	SSM( sci, SCI_TOGGLEFOLD, line, 1);
}

gboolean
sci_get_fold_expanded(ScintillaObject* sci, gint line)
{
	return SSM( sci, SCI_GETFOLDEXPANDED, line, 0);
}

void
sci_colourise( ScintillaObject* sci, gint start, gint end)
{
	SSM( sci, SCI_COLOURISE, start, end);
}
