/*  cssed (c) Iago Rubio 2003, 2004 - A tiny CSS editor.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#define SSM(s, m, w, l) scintilla_send_message(s, m, w, l)

enum {
	CSSED_FILETYPE_CSS,
	CSSED_FILETYPE_HTML,
	CSSED_FILETYPE_TEXT,
	CSSED_FILETIPE_BYNARY,
	CSSED_FILETYPE_UNKNOW
};

typedef struct _cssed_doc
{
	GtkWidget* editor;		// scintilla editor
	gint scid;				// scintilla id
	GtkWidget* notebook; 	// main notebook where this doc is attached
	gchar* filename;		// full path to file, it's dinamically allocated so dispose mem when not needed
	gint filetype;			// one of the supported filetype css, html, text, bynary or unknow
	gboolean modified; 		// modified flag, will be updated when the document changes
	gboolean autocompletion;// set autocompletion mode for the document
	Tcssed_window* window;	// main window object
} Tcssed_doc;

typedef struct _cssed_fileselector_data {
	Tcssed_window* main_window;
	GtkWidget* notebook;
	GtkWidget* fileselector;		
} Tcssed_fileselector_data;

typedef struct _cssed_cssdialogs_data {
	Tcssed_window* main_window;
	gchar* previous_string;
	gboolean add_property_string;
} Tcssed_cssdialog_data;

typedef struct _cssed_popmenu_data {
	Tcssed_window* main_window;
	Tcssed_doc* document;
	gchar* attribute;
	gchar* property; // previous_string
	gboolean add_property_string;
} Tcssed_popmenu_data;

typedef struct _cssed_point {
	Tcssed_window* main_window;
	gint x;
	gint y;
} Tcssed_point;

GtkWidget *
create_initial_blank_editor  (Tcssed_window* window);

void
create_and_attach_new_doc(Tcssed_window* window ,const gchar* title);

Tcssed_doc *
cssed_create_new_doc (Tcssed_window* window );

void
document_save(Tcssed_doc* doc);

void
create_and_attach_new_named_doc_from_fileselector_cb (GtkButton *button,   gpointer user_data);

void
document_add_text_line( Tcssed_doc * doc, char* text );

void 
document_scroll_to_caret( Tcssed_doc* doc );

Tcssed_doc*
document_get_from_notebook_page_num(Tcssed_window* window, gint page_number);

Tcssed_doc*
document_get_current(Tcssed_window* window);

gboolean
document_get_modified( Tcssed_doc* doc );

gboolean
document_dispose( Tcssed_doc* doc );

gboolean 
document_get_line_numbers_visible( Tcssed_doc* doc );

void 
document_set_line_numbers_visible( Tcssed_doc* doc, gboolean visible );
gint
document_get_position_from_line( Tcssed_doc* doc, gint line);
void
document_set_white_spaces_visible( Tcssed_doc* doc, gboolean visible);

gboolean
document_get_white_spaces_visible( Tcssed_doc* doc);

void 
document_set_css_style(Tcssed_doc* doc);

void
document_set_font_by_name(Tcssed_doc* doc, gchar* font_name);

void
document_set_font(Tcssed_doc* doc);

void
document_set_font_size(Tcssed_doc* doc, gint size);

void
document_set_current_font_size(Tcssed_doc* doc);

void
document_set_default_configuration( Tcssed_doc* doc );

void 
document_undo(Tcssed_doc* doc);

void 
document_redo(Tcssed_doc* doc);

void 
document_empty_undo_buffer(Tcssed_doc* doc);

void 
document_end_undo_action(Tcssed_doc* doc);

void 
document_new_undo_action(Tcssed_doc* doc);

void 
document_set_undo_collection(Tcssed_doc* doc, gboolean set);

gboolean
document_get_undo_collection(Tcssed_doc* doc);

void 
document_start_undo_action (Tcssed_doc* doc);

void // no wrapper function for this
document_search_text( Tcssed_doc* doc, gchar* text, gint flags );

void 
document_zoom_in(Tcssed_doc* doc);

void 
document_zoom_off(Tcssed_doc* doc);

void 
document_zoom_out(Tcssed_doc* doc);

void 
document_set_marker_at_line( Tcssed_doc* doc, gint line, gboolean set, gint marker);

gboolean
document_is_marker_set_at_line( Tcssed_doc* doc, gint line, gint marker);

gint
document_get_line_from_position( Tcssed_doc* doc, gint position);

gint 
document_get_current_position	(Tcssed_doc* doc );

void
document_set_current_position	(Tcssed_doc* doc, gint position);

void 
document_paste (Tcssed_doc* doc);

void 
document_copy (Tcssed_doc* doc);

void 
document_cut (Tcssed_doc* doc);

gint
document_get_selection_end(Tcssed_doc* doc);

gint
document_get_selection_start(Tcssed_doc* doc);

void
document_set_selection_start(Tcssed_doc* doc, gint position);

void
document_set_selection_end(Tcssed_doc* doc, gint position);

gint 
document_get_line_end_from_position(Tcssed_doc* doc, gint position);

void 
document_set_selection_range(Tcssed_doc* doc,gint start, gint end);

void
document_replace_sel(Tcssed_doc* doc, gchar* text);

void
document_clear_sel(Tcssed_doc* doc);

gint
document_get_length	(Tcssed_doc* doc);

void
document_get_text(Tcssed_doc* doc, gint len, gchar* text);

void
document_set_modified( Tcssed_doc* doc , gboolean modified);

void 
document_write_error_to_program_output(	Tcssed_doc* doc, 
										gchar* error_message,
										gchar* color);
										
void 
document_show_autocompletion_list(Tcssed_doc* doc, gint identifier, gchar* list);

gboolean
document_current_pos_is_into_selector(Tcssed_doc* doc);

gint 
document_get_text_line(Tcssed_doc* doc, gint line, gchar* buffer);

gint 
document_get_line_length(Tcssed_doc* doc, gint line);

gint
document_get_line_count( Tcssed_doc* doc );

void 
fire_dialog_from_strings_or_write_output(	Tcssed_window* window,
											Tcssed_doc* tmpdoc, 
											gchar* attribute, 
											gchar* property,
											gboolean add_previous);
											
void
document_toggle_fold_at_line(Tcssed_doc* doc, gint line);

gboolean
document_get_line_is_visible(Tcssed_doc* , gint line);

void
document_ensure_line_is_visible( Tcssed_doc* , gint line);

gint
document_get_fold_level( Tcssed_doc* , gint line);

void
create_and_attach_new_named_doc(Tcssed_window* window, gchar* filename);

void 
document_set_folding_margin_visible( Tcssed_doc* doc, gboolean visible );

gboolean 
document_get_folding_margin_visible( Tcssed_doc* doc );

gboolean
document_get_folding_enabled( Tcssed_doc* doc );

void
document_fold_all( Tcssed_doc* doc );

void
document_unfold_all( Tcssed_doc* doc );

gboolean 
document_get_fold_expanded_at_line( Tcssed_doc* doc, gint line );

void				
document_style_clear_all(Tcssed_doc* doc) ;

void				
document_colourise_all(Tcssed_doc* doc);

void				
document_colourise(Tcssed_doc* doc, gint start, gint end);

void
document_set_validator_arrow_at_line( Tcssed_doc* doc, gint line );

void
document_clear_validator_arrow( Tcssed_doc* doc );
