/*  cssed (c) Iago Rubio, 2003
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "cssedwindow.h"
#include "configparser.h"
#include "interface.h"
#include "document.h"
#include "support.h"
#include "utils.h"



int
main (int argc, char *argv[])
{
	GtkWidget *window;	            // the main window
	GList *document_list = NULL;	// the document list
	gchar *keywords_list;	    	// the CSS definition keywords (view xmlparser.c)
	Tcssed_window *cssedwin;	    // the main window object
	Tcssed_dyn_menus *cssedmenus;	// store pointers ONLY to the dinamic menu items
	Tcssed_cfg* configuration;		// configuration data
	int i;							// to parse arguments
	
	keywords_list = g_strdup(""); // empty allocated keyword list

	cssedwin = g_malloc (sizeof (Tcssed_window));
	cssedmenus = g_malloc (sizeof (Tcssed_dyn_menus));
	configuration = g_malloc (sizeof (Tcssed_cfg));

	cssedwin->keywords_list = keywords_list;
	cssedwin->document_list = document_list;
	cssedwin->dynamic_menus = cssedmenus;
	cssedwin->config = configuration;

#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif

	gtk_set_locale ();
	gtk_init (&argc, &argv);
	gdk_rgb_init ();
	
	init_default_config( cssedwin );
	parse_configuration_file ( cssedwin );
	
	add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");
	window = create_window (cssedwin);
	gtk_widget_show (window);
	
	/* args processing everything passed in taken as a file name */	
	for( i=1;i<argc;i++){
		create_and_attach_new_named_doc( cssedwin, argv[i] );
	}

	gtk_main ();
	g_free( configuration );
	g_free (cssedmenus);
	//g_list_free(cssedwin->document_list);	// the document list
	g_free(keywords_list);
	g_free (cssedwin);
	return 0;
}
