/*  cssed (c) Iago Rubio 2003-2004
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <expat.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gtk/gtk.h>

#ifdef WITH_TERMINAL
 #include <vte/vte.h>
#endif

#include "cssedwindow.h"
#include "xmlparser.h"
#include "callbacks.h"

//#define DEBUG

/* old spat stuff needed for BSD like machines as MacOsX*/
#ifndef XML_STATUS_OK
   #define XML_STATUS_OK    1
   #define XML_STATUS_ERROR 0
#endif

/* protos */
static void
fill_tree_view_start_tag_cb (void* data, const char *el, const char **attr);
static void
fill_tree_view_end_tag_cb (void *data, const char *el);

/* globals */
int depth;

/* parse the css definition, it don't validates against the DTD
	so ANY well formed XML will be parsed. Take in account that
	you can put garbled values on the treeview if you parse
	a not cssed related XML doc, resulting in a real mess !!
*/
void
parse_css_definition_and_fill_tree_view(Tcssed_tp_data* data)
{
	Tcssed_window* window;
	struct stat _stat;
	int _retval;
	void* map;
	int filedes;
	gchar* home;
	gchar* userfile;
	gchar* globalfile;
	gchar* systemfile;
	GtkTreeView* treeview ;
	gchar* err ;
	XML_Parser p;
	
	window = CSSED_WINDOW(data->window);
	treeview = GTK_TREE_VIEW( window->tree_view_css_properties );
	globalfile = PACKAGE_DATA_DIR "/" PACKAGE "/data/cssed-def.xml";
	// check if the user have a local definition file
	home = getenv("HOME");
	if( home != NULL ){
		userfile = g_strconcat(home,"/.cssed/cssed-def.xml",NULL);
		if( g_file_test(userfile,G_FILE_TEST_IS_REGULAR| G_FILE_TEST_EXISTS) ){
			systemfile = userfile;
		}else{
			systemfile = globalfile;
		}
	}else{
		systemfile = globalfile;
	}
	
#ifdef DEBUG
	g_print("Will try to use: %s\n", systemfile );
#endif	
	
	_retval = stat( systemfile , &_stat );
	
	if( _retval != 0 ){
		cssed_error_message( "Can't stat on CSS definitions file !!!" , "Error" );
  	}else{
		if( S_ISREG( _stat.st_mode ) ){
	  		if( (filedes = open(systemfile, O_RDONLY)) < 0){
				cssed_error_message( "Can't open CSS defitions file !!!" , "Error" );
	  		}else{
				if( (map = mmap(0,_stat.st_size, PROT_READ, MAP_SHARED, filedes, 0)) == MAP_FAILED ){
		  			cssed_error_message( "Can't map CSS defitions file to memory !!!" , "Error" );
		  			close(filedes);
					return;
				}else{
					p = XML_ParserCreate (NULL);
					if (!p)
					{
						cssed_error_message( "Can't allocate memory for XML parser !!!" , "Error" );
						munmap(map,_stat.st_size );
						close(filedes);
						return;
					}
					
					XML_SetElementHandler (p, fill_tree_view_start_tag_cb, fill_tree_view_end_tag_cb);
					XML_SetUserData( p, (void *) data); 

					if (XML_Parse (p, (char*) map, _stat.st_size, 1) == XML_STATUS_ERROR)
					{				
						err = g_strdup_printf("The CSS definiton XML file is bogus,\nParse error at line %d:\n%s\n",
													XML_GetCurrentLineNumber (p),
						 							XML_ErrorString (XML_GetErrorCode (p)) );                                            			
						cssed_error_message	(err,"Configuration error" );
						g_free(err);
					}
				}
				munmap(map,_stat.st_size);
				close(filedes);
			}
		}
	}
	if( userfile ) g_free(userfile);
}

static void
fill_tree_view_start_tag_cb (void* data, const char *el, const char **attr)
{
	gint i;
	gchar* err_depth;
	GtkWidget* tree;
	GtkTreeStore *model;
	GtkWidget *treeview;
	GtkTreeIter* toplevel, *child, *third;
	Tcssed_window* window;
	Tcssed_tp_data* tp_data = (Tcssed_tp_data*) data;
	gchar* keywordslist;
	gchar* key_copy;
	
	window = CSSED_WINDOW(tp_data->window);	
	
	toplevel =	tp_data->first_col;
	child =  	tp_data->second_col;
	third =  	tp_data->thrid_col;
	model =  	tp_data->store;	
	
	tree = GTK_WIDGET( window->tree_view_css_properties );
	
	switch( depth ){
		case 0: // in depth 0 we've got the imp def and name
				for (i = 0; attr[i]; i += 2)
				{
					if( strcmp( attr[i], "implementation")==0){						
						gtk_tree_store_append (model, toplevel, NULL);
						gtk_tree_store_set(model, toplevel, depth, attr[i + 1], -1);
					}
				}		
		break;
		case 1: // in depth 1 will find reserved words ( properties )
				for (i = 0; attr[i]; i += 2)
				{
					if( strcmp( attr[i], "name")==0){	
						gtk_tree_store_append (model, child, toplevel);
						gtk_tree_store_set (model, child, depth, attr[i + 1] , -1);
						// add to keywordlist
						key_copy = g_strdup( window->keywords_list );
						g_free( window->keywords_list );
						window->keywords_list = g_strjoin (" ",key_copy, attr[i + 1], NULL );
						g_free( key_copy );
					}
				}
		break;
		case 2: // in depth 3 there are the properties valid values
				for (i = 0; attr[i]; i += 2)
				{	
					if( strcmp( attr[i], "name")==0){						
						gtk_tree_store_append (model, third, child);
						gtk_tree_store_set (model, third, depth, attr[i + 1], -1);
					}
				}
		break;
		default:
				// it must go out here
				err_depth = g_strdup_printf("Bad XML definition file\ndepth %d at element %s\nMax deptph must be 3", depth, el);
				cssed_error_message (err_depth, "Configuration file error");
				g_free(err_depth);
		break;
	}
	depth++;
}

static void
fill_tree_view_end_tag_cb (void *data, const char *el)
{
	depth--;
}
